-- Player bars
--[[
Drathals HUD2 (c) 2007 by Horacio Hoyos / Arcanefoam
Drathals HUD (c) 2006 by Caeryn Dryad
Drathals HUD (c) 2006 by Markus Inger / Drathal / Silberklinge / Silbersegen

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are 
met:

    * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above 
       copyright notice, this list of conditions and the following 
       disclaimer in the documentation and/or other materials provided 
       with the distribution.
    * Neither the name of the DHUD2 Development Team nor the names of 
       its contributors may be used to endorse or promote products derived 
       from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]
local DHUD2, self = DHUD2, DHUD2
local _G = _G
local string_find = string.find
local string_sub = string.sub 
local math_floor = math.floor
local castTimer = 0
local spellDuration, isChanneling, barFadeOut, barFlash, finishTime, fadeAlpha
local rBorder = { "h","h","h","h","h", }
local lBorder = { "h","h","h","h","h", }

local setBarHeight, setBarColor, flashCastBar, fadeoutCastBar
do
    function setBarHeight(bar, percent)
        -- Hide when Bar empty 
        if math_floor(percent * 100) == 0 then
            return false
        end
        local texture = _G["DHUD2_"..bar.."_Texture"]
        local point, parent, relative, x, y, width, height, x0, x1, y0, y1, path, tex_height, tex_gap_top, tex_gap_bottom = unpack( self.layout.bars[bar] )        
        local tex_gap_top_p    = tex_gap_top / tex_height
        local tex_gap_bottom_p = tex_gap_bottom / tex_height
        local h = (tex_height - tex_gap_top - tex_gap_bottom) * percent
        local top    = 1-(percent-(tex_gap_top_p))
        local bottom = 1-tex_gap_bottom_p
        top = top  - ((tex_gap_top_p+tex_gap_bottom_p)*(1-percent))
        texture:SetHeight(h)
        texture:SetTexCoord(x0, x1, top, bottom )
        texture:SetPoint(point, _G["DHUD2_"..bar], relative, x, tex_gap_bottom)
        texture:Show()
        return true
    end 
    
    function setBarColor(bar, color)
        local texture = _G["DHUD2_"..bar.."_Texture"]
        texture:SetVertexColor(color.r, color.g, color.b)
    end
    
    function addBarBorder(bar, show)
        if self.db.profile.show.barBorders then
            if string_find(bar, "p") then
                -- Is a Pet bar
                if string_find(bar, "l") then
                    if string_find(bar, "o") then
                        lBorder[4] = show and "s" or "h"
                    else
                        lBorder[5] = show and "s" or "h"
                    end
                else
                    if string_find(bar, "o") then
                        rBorder[4] = show and "s" or "h"
                    else
                        rBorder[5] = show and "s" or "h"
                    end
                end
            else
                -- Is a target bar
                if string_find(bar, "l") then
                    if string_find(bar, "o") then
                        lBorder[1] = show and "s" or "h"
                    elseif string_find(bar, "m") then
                        lBorder[2] = show and "s" or "h"
                    else
                        lBorder[3] = show and "s" or "h"
                    end
                else
                    if string_find(bar, "o") then
                        rBorder[1] = show and "s" or "h"
                    elseif string_find(bar, "m") then
                        rBorder[2] = show and "s" or "h"
                    else
                        rBorder[3] = show and "s" or "h"
                    end
                end
            end
        else
            lBorder[1] = "h"
            lBorder[2] = "h"
            lBorder[3] = "h"
            lBorder[4] = "h"
            lBorder[5] = "h"
            rBorder[1] = "h"
            rBorder[2] = "h"
            rBorder[3] = "h"
            rBorder[4] = "h"
            rBorder[5] = "h"
        end
    end

    function flashCastBar()
        local alpha = DHUD2_flash:GetAlpha() + CASTING_BAR_FLASH_STEP
        DHUD2_flash:SetAlpha(alpha)
        DHUD2_flash:Show()
        barFlash = true
        fadeAlpha = 0
        DHUD2_flash:SetScript("OnUpdate", self.OnUpdateFlash)
    end

    function fadeoutCastBar()
        --local alpha = DHUD2_flash:GetAlpha() + CASTING_BAR_FLASH_STEP
        --DHUD2_flash:SetAlpha(alpha)
        --DHUD2_flash:Show()
        barFadeOut = true
        fadeAlpha = 1
        --DHUD2_flash:SetScript("OnUpdate", self.OnUpdateFlash)
    end
end

function DHUD2:updateBar(bar, maxval, curval, fracval, color)
    self:Debug("updateBar", bar, maxval, curval, fracval, color)
    if setBarHeight(bar, fracval) then
        _G["DHUD2_"..bar]:Show()
        setBarColor(bar, color)
        addBarBorder(bar, true)
        self:updateBarText(bar, maxval, curval, fracval, color.hex)
    else
        addBarBorder(bar, false)
        self:hideBarText(bar)
        _G["DHUD2_"..bar]:Hide()
    end
end

function DHUD2:initBorders()
    local i
    for i = 1, 5 do
        lBorder[i] = "h"
        rBorder[i] = "h"
    end
end


function DHUD2:addBackgroundBorder()
    -- Border is in the form 321pp, where each one has a value of s (show) or h (hide). This information is saved in a list (lBorder o rBorder) and contatenated to creat the path of the texture.
    local p = self.db.profile
    if p.show.barBorders then
        local path = ""
        for i,v in ipairs(lBorder) do path = path..v end
        self:Debug("addBackgroundBorder->left", path)
        if path == "hhhhh" then
            DHUD2_leftFrame_Texture:SetTexture(nil)
        else
            texture = "Interface\\AddOns\\DHUD2\\textures\\"..path
            DHUD2_leftFrame_Texture:SetTexture(texture)
            DHUD2_leftFrame_Texture:SetTexCoord(0,1,0,1)
        end
        path = ""
        for i,v in ipairs(rBorder) do path = path..v end
        self:Debug("addBackgroundBorder->right", path)
        if path == "hhhhh" then
            DHUD2_rightFrame_Texture:SetTexture(nil)
        else
            texture = "Interface\\AddOns\\DHUD2\\textures\\"..path
            DHUD2_rightFrame_Texture:SetTexture(texture)
            DHUD2_rightFrame_Texture:SetTexCoord(1,0,0,1)
        end
    else
        DHUD2_leftFrame_Texture:SetTexture(nil)
        DHUD2_rightFrame_Texture:SetTexture(nil)
    end
end

function DHUD2:OnShow(this)
    castTimer = 0
    DHUD2_casting:SetScript("OnUpdate", OnUpdateCast)
end

function DHUD2:OnHide(this)
    castTimer = nil
    DHUD2_casting:SetScript("OnUpdate", nil)
end

function DHUD2:OnUpdateCast(this, elapsed)
    local p = self.db.profile
    castTimer = castTimer + elapsed
    if isCasting then
        local value = castTimer / (spellDuration + delayTime) 
        DHUD2_casting_Texture:SetVertexColor(colorize("casting", "cast", value))
        if p.reversecasting then
            cast = (spellDuration + delayTime) - castTimer
            setBarHeight("casting", 1 - value)        
        else
            cast = castTimer
            setBarHeight("casting", value)
        end            
        updateCastText(cast)
    elseif isChanneling then
        local value = castTimer / (spellDuration + delayTime)
        setBarHeight("casting", value )
        DHUD2_casting_Texture:SetVertexColor(colorize("casting", "channel",value))
        updateCastText(castTimer)        
    elseif barFadeOut then
        fadeAlpha = this:GetAlpha() - CASTING_BAR_ALPHA_STEP
        if fadeAlpha > 0 then
            this:SetAlpha(fadeAlpha)
            DHUD2_castTime_Text:SetAlpha(fadeAlpha)
            DHUD2_castDelay_Text:SetAlpha(fadeAlpha)
        else
            barFadeOut = false
            this:SetScript("OnUpdate", nil)
            this:Hide()
            this:SetAlpha(1)
            DHUD2_castTime_Text:SetAlpha(1)
            DHUD2_castDelay_Text:SetAlpha(1)
            delayTime = 0
            updateCastText(0)
            updateAlpha()
        end 
    end
end

function DHUD2:OnUpdateFlash(this)
    if barFlash then
        fadeAlpha = this:GetAlpha() + CASTING_BAR_FLASH_STEP
        if fadeAlpha < 1 then
            this:SetAlpha(fadeAlpha)
        else
            barFlash = false            
            this:SetScript("OnUpdate", nil)
            this:Hide()
            this:SetAlpha(1)
            updateAlpha()
        end            
    end
end

function DHUD2:positionFrame(name, x2, y2)
    local p = self.db.profile
    local xn , yn, mx, my = unpack ( p.moveFrame[name])
    local x2 = tonumber(config[xn] or 0)
    local y2 = tonumber(config[yn] or 0)
    if mx == "-" then
        x2 = 0 - x2
    end
    if my == "-" then
        y2 = 0 - y2
    end
    local ref = ""
    local point, parent, relative, x, y, width, height = unpack( self.Layouts[p.layout].mainFrame )        
    if name == "mainFrame" then        
        ref = _G["DHUD2_"..name]
        ref:SetPoint(point, parent, relative, x + x2, y + y2)
        return
    elseif string_find(name, "Frame") then
        point, parent, relative, x, y, width, height = unpack( self.Layouts[p.layout].frames[name] )
        ref = _G["DHUD2_"..name]
    else
        point, parent, relative, x, y, width, height = unpack( self.Layouts[p.layout].text[name] )
        ref = _G["DHUD2_"..name.."_Text"]
    end
    ref:SetPoint(point, "DHUD2_"..parent, relative, x + x2, y + y2)
end
